#!/usr/bin/perl -w

# This is a simple Auto-Responder which can be installed onto any *ix (Linux is tested)
# installation within /etc/aliases:
# echo: "|/etc/postfix/mailecho.pl"
# if you use postfix don't forget to do postalias /etc/aliases after that
#
# For logging (and rate-limitting which depends on) you need a directory where
# the user nobody:nogroup has write permissions (maybe /tmp?) and the files are
# not deleted every few seconds
#
# Most of configuration is done in a config-file
#
# you need to install the per modules Net::SMTP and Email::Date
# and the small tool 'figlet'
#

use strict;
use Net::SMTP;
use Email::Date;

# Import config-File
my $config=do("/etc/postfix/mailecho_config.pl");
die "Error parsing config file: $@" if $@;
die "Error reading config file: $!" unless defined $config;

my $smtp;

my $FROM="";
my $FROMDOMAIN="";
my $TO="";
my $REPLY;
my $SUBJECT="";
my $HEADER=1;
my $DOMAIN="";
my $USER="";
my $MAIL="";
my $LINENUM=0;
my $USERFROM;
my $DONTANSWER=0;
my $SPFPASSED=0;
my @BANNEDDOMAINS=("stream", "today", "ru", "xxx");

# create logfile if not found
my $logfile=($config->{logfile});
my %last;
unless ( -e $logfile) {
  open my $fc, ">", $logfile;
  close $fc;
}

# read all entries from logfile within the last 'ratelimit' seconds into a hash
my $now=time();
my $minage=$now-($config->{ratelimit});
open my $in, '<', $logfile or die $!;
while (my $line = <$in>) {
  chomp $line;
  my ($timestamp,$address,$service) = split/\s+/, $line;
  if($timestamp =~ /^[0-9]+$/) {
    if($timestamp > $minage) {
      $last{$address}=$timestamp;
    }
  }
}
close $in;

#while ( (my $k, my $v) = each %last ) {
#    print "$k => $v\n";
#}
#$DONTANSWER=1;

# Parse Mail-Headers and copy the lines of header into a variable $MAIL
while (<>){
  if ($HEADER == 1){
    if ($LINENUM != 0){
      $MAIL.="> ".$_;
    }
    $LINENUM++;
    # Search for SPF Check header
    if ($_ =~ /^Received-SPF: Pass \(mailfrom\)/){
      $SPFPASSED=1;
    }
    # Tolerate Telekom Mails for the moment...
    if ($_ =~ /^Received-SPF: None \(mailfrom\).*helo=mailout[0-9]+.telekom.de.*/){
      $SPFPASSED=1;
    }
    if ($_ =~ /^[Ff][Rr][Oo][Mm]:/){
      $FROM=$_;
      $FROM =~ s/^.*://;
    }
    if ($_ =~ /^[Tt][Oo]:/){
      $TO=$_;
      $TO=~ s/^.*://;
    }
    if ($_ =~ /^[Ss][Uu][Bb][Jj][Ee][Cc][Tt]:/){
      $SUBJECT = $_;
      $SUBJECT =~ s/^Subject: /Re: /;
    }
    if ($_ =~ /^[Rr][Ee][Pp][Ll][Yy]-[Tt][Oo]:/){
      $REPLY=$_;
      $REPLY =~ s/^.*://;
    }
  }
  $HEADER=0 if $_ =~ /^$/;
}

$FROM=$REPLY if defined $REPLY;

$FROM =~ s/^.*<//;
$FROM =~ s/>.*//;
$FROM =~ s/ //g;
$TO =~ s/^.*<//;
$TO =~ s/>.*//;
$TO =~ s/ //g;
$DOMAIN = $TO;
$DOMAIN =~ s/.*@//;
$USER = $TO;
$USER =~ s/@.*//;
$USERFROM = $FROM;
$USERFROM =~ s/@.*//;
$FROMDOMAIN = $FROM;
$FROMDOMAIN =~ s/.*@//;
$FROMDOMAIN = lc $FROMDOMAIN;

chomp ($FROM);
chomp ($DOMAIN);
chomp ($USER);
chomp ($SUBJECT);

foreach (@BANNEDDOMAINS)
{
  if ($FROMDOMAIN =~ /($_)$/)
  {
    $DONTANSWER=1;
    open my $out, '>>', $logfile;
    print $out time(), " ", $FROM, " ", $USER."@".$DOMAIN, " *.".$_." BANNED\n";
    close $out;
  }
}

# dont answer if the sender uset the service in the last few minutes (ratelimit)
if ( defined $last{$FROM} ) {
  $DONTANSWER=1;
}

# we only answer if the mail is sent to a defined domain
if ($DOMAIN !~ /^($config->{domainlist})$/){
  $DONTANSWER=1;
}

# we only answer if the mail is sent to echo of felswand@
if ($USER !~ /(echo|felswand)/){
  $DONTANSWER=1;
}

# Avoid PingPong - if sender-address contains 'echo' or 'felswand' or 'MAILER-DAEMON'
if ($USERFROM =~ /(echo|felswand|MAILER-DAEMON)/){
  $DONTANSWER=1;
  open my $out, '>>', $logfile;
  print $out time(), " ", $FROM, " ", $USER."@".$DOMAIN, " ", $USERFROM."@* BANNED\n";
  close $out;
}

# Block Mails with no SPF
if ($SPFPASSED == 0){
  $DONTANSWER=1;
  open my $out, '>>', $logfile;
  print $out time(), " ", $FROM, " ", $USER."@".$DOMAIN, " SPF Check not passed\n";
  close $out;
}

# Create the Mail body
$MAIL="Sie haben erfolgreich unseren \"".$USER."@".$DOMAIN."\" Dienst erreicht.

You have successfully reached the ".$USER." service of\n".`figlet $DOMAIN`."
".$DOMAIN."

The following message (only headers) arrived at the ".$USER." service:

".$MAIL;

# Send the answer
if ($FROM =~ /@.*\..+/){
  if ($DONTANSWER == 0){
    open my $out, '>>', $logfile;
    print $out time(), " ", $FROM, " ", $USER."@".$DOMAIN, "\n";
    close $out;
    $smtp = Net::SMTP->new($config->{smtpserver}, Hello => $config->{myhostname}, Port => $config->{smtpport});
    $smtp->mail($USER."@".$DOMAIN);
    $smtp->to($FROM);
    $smtp->data();
    $smtp->datasend("To: ".$FROM."\n");
    $smtp->datasend("Subject: ".$SUBJECT."\n");
    $smtp->datasend("From: ".$DOMAIN." ".$USER." service <".$USER."-reply@".$DOMAIN.">\n");
    $smtp->datasend("Date: ".format_date."\n");
    $smtp->datasend("\n");
    $smtp->datasend($MAIL);
    $smtp->dataend();
    $smtp->quit;
  }
}
